var _inherits = require('babel-runtime/helpers/inherits').default;

var _classCallCheck = require('babel-runtime/helpers/class-call-check').default;

var _regeneratorRuntime = require('babel-runtime/regenerator').default;

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default').default;

var _interopRequireWildcard = require('babel-runtime/helpers/interop-require-wildcard').default;

exports.__esModule = true;

var _router = require('./router');

var _router2 = _interopRequireDefault(_router);

var _http = require('http');

var _http2 = _interopRequireDefault(_http);

var _utilsUrl = require('../utils/url');

var urlUtils = _interopRequireWildcard(_utilsUrl);

var _readFileRelative = require('read-file-relative');

var _utilsHttp = require('../utils/http');

var _upload = require('../upload');

var _requestPipeline = require('../request-pipeline');

var _shadowUiCreateShadowStylesheet = require('../shadow-ui/create-shadow-stylesheet');

var _shadowUiCreateShadowStylesheet2 = _interopRequireDefault(_shadowUiCreateShadowStylesheet);

// Const
var CLIENT_SCRIPT = _readFileRelative.readSync('../client/hammerhead.js');

// Static
function parseServiceMsg(body) {
    body = body.toString();

    try {
        return JSON.parse(body);
    } catch (err) {
        return null;
    }
}

function createServerInfo(hostname, port, crossDomainPort) {
    return {
        hostname: hostname,
        port: port,
        crossDomainPort: crossDomainPort,
        domain: 'http://' + hostname + ':' + port
    };
}

// Proxy

var Proxy = (function (_Router) {
    _inherits(Proxy, _Router);

    function Proxy(hostname, port1, port2) {
        var _this = this;

        _classCallCheck(this, Proxy);

        _Router.call(this);

        this.openSessions = {};

        this.server1Info = createServerInfo(hostname, port1, port2);
        this.server2Info = createServerInfo(hostname, port2, port1);
        this.server1 = _http2.default.createServer(function (req, res) {
            return _this._onRequest(req, res, _this.server1Info);
        });
        this.server2 = _http2.default.createServer(function (req, res) {
            return _this._onRequest(req, res, _this.server2Info);
        });

        this.server1.listen(port1);
        this.server2.listen(port2);

        this.sockets = [];

        // BUG: GH-89
        this._startSocketsCollecting();
        this._registerServiceRoutes();
    }

    Proxy.prototype._closeSockets = function _closeSockets() {
        this.sockets.forEach(function (socket) {
            return socket.destroy();
        });
    };

    Proxy.prototype._startSocketsCollecting = function _startSocketsCollecting() {
        var _this2 = this;

        var handler = function (socket) {
            _this2.sockets.push(socket);
            socket.on('close', function () {
                return _this2.sockets.splice(_this2.sockets.indexOf(socket), 1);
            });
        };

        this.server1.on('connection', handler);
        this.server2.on('connection', handler);
    };

    Proxy.prototype._registerServiceRoutes = function _registerServiceRoutes() {
        var _this3 = this;

        this.GET('/hammerhead.js', {
            contentType: 'application/x-javascript',
            content: CLIENT_SCRIPT
        });

        this.POST('/ie9-file-reader-shim', _upload.ie9FileReaderShim);
        this.POST('/messaging', function (req, res, serverInfo) {
            return _this3._onServiceMessage(req, res, serverInfo);
        });
        this.GET('/task.js', function (req, res, serverInfo) {
            return _this3._onTaskScriptRequest(req, res, serverInfo, false);
        });
        this.GET('/iframe-task.js', function (req, res, serverInfo) {
            return _this3._onTaskScriptRequest(req, res, serverInfo, true);
        });
    };

    Proxy.prototype._onServiceMessage = function _onServiceMessage(req, res, serverInfo) {
        var body, msg, session, result;
        return _regeneratorRuntime.async(function _onServiceMessage$(context$2$0) {
            while (1) switch (context$2$0.prev = context$2$0.next) {
                case 0:
                    context$2$0.next = 2;
                    return _regeneratorRuntime.awrap(_utilsHttp.fetchBody(req));

                case 2:
                    body = context$2$0.sent;
                    msg = parseServiceMsg(body);
                    session = msg && this.openSessions[msg.sessionId];

                    if (!session) {
                        context$2$0.next = 18;
                        break;
                    }

                    context$2$0.prev = 6;
                    context$2$0.next = 9;
                    return _regeneratorRuntime.awrap(session.handleServiceMessage(msg, serverInfo));

                case 9:
                    result = context$2$0.sent;

                    _utilsHttp.respondWithJSON(res, result || '');
                    context$2$0.next = 16;
                    break;

                case 13:
                    context$2$0.prev = 13;
                    context$2$0.t0 = context$2$0['catch'](6);

                    _utilsHttp.respond500(res, context$2$0.t0.toString());

                case 16:
                    context$2$0.next = 19;
                    break;

                case 18:
                    _utilsHttp.respond500(res, 'Session is not opened in proxy');

                case 19:
                case 'end':
                    return context$2$0.stop();
            }
        }, null, this, [[6, 13]]);
    };

    Proxy.prototype._onTaskScriptRequest = function _onTaskScriptRequest(req, res, serverInfo, isIframe) {
        var referer = req.headers['referer'];
        var refererDest = referer && urlUtils.parseProxyUrl(referer);
        var session = refererDest && this.openSessions[refererDest.sessionId];

        if (session) {
            res.setHeader('content-type', 'application/x-javascript');
            res.setHeader('cache-control', 'no-cache, no-store, must-revalidate');
            res.setHeader('pragma', 'no-cache');
            res.end(session.getTaskScript(referer, refererDest.destUrl, serverInfo, isIframe, true));
        } else _utilsHttp.respond500(res);
    };

    Proxy.prototype._onRequest = function _onRequest(req, res, serverInfo) {
        // NOTE: Not a service request, execute the proxy pipeline.
        if (!this._route(req, res, serverInfo)) _requestPipeline.run(req, res, serverInfo, this.openSessions);
    };

    Proxy.prototype._processStaticContent = function _processStaticContent(handler) {
        if (handler.isShadowUIStylesheet) handler.content = _shadowUiCreateShadowStylesheet2.default(handler.content);
    };

    // API

    Proxy.prototype.close = function close() {
        this.server1.close();
        this.server2.close();
        this._closeSockets();
    };

    Proxy.prototype.openSession = function openSession(url, session) {
        session.proxy = this;
        this.openSessions[session.id] = session;

        url = urlUtils.convertHostToLowerCase(url);

        return urlUtils.getProxyUrl(url, this.server1Info.hostname, this.server1Info.port, session.id);
    };

    Proxy.prototype.closeSession = function closeSession(session) {
        session.proxy = null;
        delete this.openSessions[session.id];
    };

    return Proxy;
})(_router2.default);

exports.default = Proxy;
module.exports = exports.default;