var _regeneratorRuntime = require('babel-runtime/regenerator').default;

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default').default;

var _interopRequireWildcard = require('babel-runtime/helpers/interop-require-wildcard').default;

exports.__esModule = true;
exports.run = run;

var _destinationRequest = require('./destination-request');

var _destinationRequest2 = _interopRequireDefault(_destinationRequest);

var _context = require('./context');

var _context2 = _interopRequireDefault(_context);

var _headerTransforms = require('./header-transforms');

var headerTransforms = _interopRequireWildcard(_headerTransforms);

var _processingResources = require('../processing/resources');

var _messages = require('../messages');

var _connectionResetGuard = require('./connection-reset-guard');

var _connectionResetGuard2 = _interopRequireDefault(_connectionResetGuard);

var _xhrSameOriginPolicy = require('./xhr/same-origin-policy');

var _utilsHttp = require('../utils/http');

var _upload = require('../upload');

// Stages
var stages = {
    0: function fetchProxyRequestBody(ctx, next) {
        return _regeneratorRuntime.async(function fetchProxyRequestBody$(context$1$0) {
            while (1) switch (context$1$0.prev = context$1$0.next) {
                case 0:
                    context$1$0.next = 2;
                    return _regeneratorRuntime.awrap(_utilsHttp.fetchBody(ctx.req));

                case 2:
                    ctx.reqBody = context$1$0.sent;

                    next();

                case 4:
                case 'end':
                    return context$1$0.stop();
            }
        }, null, this);
    },

    1: function sendDestinationRequest(ctx, next) {
        var opts = createReqOpts(ctx);
        var req = new _destinationRequest2.default(opts);

        req.on('response', function (res) {
            ctx.destRes = res;
            next();
        });

        req.on('error', function () {
            return ctx.hasDestReqErr = true;
        });
        req.on('fatalError', function (err) {
            return error(ctx, err);
        });
    },

    2: function checkSameOriginPolicyCompliance(ctx, next) {
        if (ctx.isXhr && !_xhrSameOriginPolicy.check(ctx)) {
            ctx.closeWithError(0);
            return;
        }

        next();
    },

    3: function decideOnProcessingStrategy(ctx, next) {
        ctx.buildContentInfo();

        if (ctx.contentInfo.requireProcessing && ctx.destRes.statusCode === 204) ctx.destRes.statusCode = 200;

        // NOTE: Just pipe the content body to the browser if we don't need to process it.
        if (!ctx.contentInfo.requireProcessing) {
            sendResponseHeaders(ctx);
            ctx.destRes.pipe(ctx.res);
            return;
        }

        next();
    },

    4: function fetchContent(ctx, next) {
        return _regeneratorRuntime.async(function fetchContent$(context$1$0) {
            while (1) switch (context$1$0.prev = context$1$0.next) {
                case 0:
                    context$1$0.next = 2;
                    return _regeneratorRuntime.awrap(_utilsHttp.fetchBody(ctx.destRes));

                case 2:
                    ctx.destResBody = context$1$0.sent;

                    if (!(ctx.hasDestReqErr && isDestResBodyMalformed(ctx))) {
                        context$1$0.next = 6;
                        break;
                    }

                    error(_messages.getText(_messages.MESSAGE.destConnectionTerminated, ctx.dest.url));

                    return context$1$0.abrupt('return');

                case 6:

                    next();

                case 7:
                case 'end':
                    return context$1$0.stop();
            }
        }, null, this);
    },

    5: function processContent(ctx, next) {
        return _regeneratorRuntime.async(function processContent$(context$1$0) {
            while (1) switch (context$1$0.prev = context$1$0.next) {
                case 0:
                    context$1$0.prev = 0;
                    context$1$0.next = 3;
                    return _regeneratorRuntime.awrap(_processingResources.process(ctx));

                case 3:
                    ctx.destResBody = context$1$0.sent;

                    next();
                    context$1$0.next = 10;
                    break;

                case 7:
                    context$1$0.prev = 7;
                    context$1$0.t0 = context$1$0['catch'](0);

                    error(ctx, context$1$0.t0);

                case 10:
                case 'end':
                    return context$1$0.stop();
            }
        }, null, this, [[0, 7]]);
    },

    6: function sendProxyResponse(ctx) {
        sendResponseHeaders(ctx);

        _connectionResetGuard2.default(function () {
            ctx.res.write(ctx.destResBody);
            ctx.res.end();
        });
    }
};

// Utils
function createReqOpts(ctx) {
    var bodyWithUploads = _upload.inject(ctx.req.headers['content-type'], ctx.reqBody);

    // NOTE: First, we should rewrite the request body, because the 'content-length' header will be built based on it.
    if (bodyWithUploads) ctx.reqBody = bodyWithUploads;

    // NOTE: All headers, including 'content-length', are built here.
    var headers = headerTransforms.forRequest(ctx, this);

    return {
        url: ctx.dest.url,
        protocol: ctx.dest.protocol,
        hostname: ctx.dest.hostname,
        host: ctx.dest.host,
        port: ctx.dest.port,
        path: ctx.dest.partAfterHost,
        method: ctx.req.method,
        credentials: ctx.session.getAuthCredentials(),
        body: ctx.reqBody,
        headers: headers
    };
}

function sendResponseHeaders(ctx) {
    var headers = headerTransforms.forResponse(ctx);

    ctx.res.writeHead(ctx.destRes.statusCode, headers);
    ctx.res.addTrailers(ctx.destRes.trailers);
}

function error(ctx, err) {
    if (ctx.isPage && !ctx.isIframe) ctx.session.handlePageError(ctx, err);else ctx.closeWithError(500, err);
}

function isDestResBodyMalformed(ctx) {
    return !ctx.destResBody || ctx.destResBody.length !== ctx.destRes.headers['content-length'];
}

// API

function run(req, res, serverInfo, openSessions) {
    var ctx = new _context2.default(req, res, serverInfo);

    if (ctx.dispatch(openSessions)) {
        var stageIdx = 0;
        var next = function () {
            return stages[++stageIdx](ctx, next);
        };

        stages[0](ctx, next);
    } else _utilsHttp.respond404(res);
}

// NOTE: Sometimes the underlying socket emits an error event. But if we have a response body,
// we can still process such requests. (B234324)