var _inherits = require('babel-runtime/helpers/inherits').default;

var _classCallCheck = require('babel-runtime/helpers/class-call-check').default;

var _regeneratorRuntime = require('babel-runtime/regenerator').default;

var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default').default;

exports.__esModule = true;

var _mustache = require('mustache');

var _mustache2 = _interopRequireDefault(_mustache);

var _nodeUuid = require('node-uuid');

var _nodeUuid2 = _interopRequireDefault(_nodeUuid);

var _readFileRelative = require('read-file-relative');

var _events = require('events');

var _cookies = require('./cookies');

var _cookies2 = _interopRequireDefault(_cookies);

var _uploadStorage = require('../upload/storage');

var _uploadStorage2 = _interopRequireDefault(_uploadStorage);

var _command = require('./command');

var _command2 = _interopRequireDefault(_command);

var _utilsUrl = require('../utils/url');

// Const
var TASK_TEMPLATE = _readFileRelative.readSync('../client/task.js.mustache');

// Session

var Session = (function (_EventEmitter) {
    _inherits(Session, _EventEmitter);

    function Session(uploadsRoot) {
        _classCallCheck(this, Session);

        _EventEmitter.call(this);

        this.uploadStorage = new _uploadStorage2.default(uploadsRoot);

        this.id = Session._generateSessionId();
        this.cookies = new _cookies2.default();
        this.proxy = null;
        this.pageLoadCount = 0;
        this.injectable = {
            scripts: ['/hammerhead.js'],
            styles: []
        };
    }

    // Service message handlers

    Session._generateSessionId = function _generateSessionId() {
        // NOTE: GH-116
        return _nodeUuid2.default.v4().substr(0, 3);
    };

    Session.prototype.handleServiceMessage = function handleServiceMessage(msg, serverInfo) {
        return _regeneratorRuntime.async(function handleServiceMessage$(context$2$0) {
            while (1) switch (context$2$0.prev = context$2$0.next) {
                case 0:
                    if (!this[msg.cmd]) {
                        context$2$0.next = 4;
                        break;
                    }

                    context$2$0.next = 3;
                    return _regeneratorRuntime.awrap(this[msg.cmd](msg, serverInfo));

                case 3:
                    return context$2$0.abrupt('return', context$2$0.sent);

                case 4:
                    throw new Error('Malformed service message or message handler is not implemented');

                case 5:
                case 'end':
                    return context$2$0.stop();
            }
        }, null, this);
    };

    Session.prototype._fillTaskScriptTemplate = function _fillTaskScriptTemplate(serverInfo, isFirstPageLoad, referer, cookie, iframeTaskScriptTemplate, payloadScript) {
        referer = referer === null ? '{{{referer}}}' : referer;
        cookie = cookie === null ? '{{{cookie}}}' : cookie;

        iframeTaskScriptTemplate = iframeTaskScriptTemplate === null ? '{{{iframeTaskScriptTemplate}}}' : iframeTaskScriptTemplate;

        return _mustache2.default.render(TASK_TEMPLATE, {
            sessionId: this.id,
            serviceMsgUrl: serverInfo.domain + '/messaging',
            ie9FileReaderShimUrl: serverInfo.domain + '/ie9-file-reader-shim',
            crossDomainPort: serverInfo.crossDomainPort,
            isFirstPageLoad: isFirstPageLoad,
            referer: referer,
            cookie: cookie,
            iframeTaskScriptTemplate: iframeTaskScriptTemplate,
            payloadScript: payloadScript
        });
    };

    Session.prototype.getIframeTaskScriptTemplate = function getIframeTaskScriptTemplate(serverInfo) {
        return JSON.stringify(this._fillTaskScriptTemplate(serverInfo, false, null, null, null, this._getIframePayloadScript(true)));
    };

    Session.prototype.getTaskScript = function getTaskScript(referer, cookieUrl, serverInfo, isIframe, withPayload) {
        var cookies = JSON.stringify(this.cookies.getClientString(cookieUrl));
        var payloadScript = '';

        if (withPayload) payloadScript = isIframe ? this._getIframePayloadScript() : this._getPayloadScript();

        var taskScript = this._fillTaskScriptTemplate(serverInfo, this.pageLoadCount === 0, referer, cookies, this.getIframeTaskScriptTemplate(serverInfo), payloadScript);

        this.pageLoadCount++;

        return taskScript;
    };

    Session.prototype._getIframePayloadScript = function _getIframePayloadScript() /* iframeWithoutSrc */{
        throw new Error('Not implemented');
    };

    Session.prototype._getPayloadScript = function _getPayloadScript() {
        throw new Error('Not implemented');
    };

    Session.prototype.handleFileDownload = function handleFileDownload() /* ctx */{
        throw new Error('Not implemented');
    };

    Session.prototype.handlePageError = function handlePageError() /* ctx, err */{
        throw new Error('Not implemented');
    };

    Session.prototype.getAuthCredentials = function getAuthCredentials() {
        throw new Error('Not implemented');
    };

    return Session;
})(_events.EventEmitter);

exports.default = Session;
var ServiceMessages = Session.prototype;

ServiceMessages[_command2.default.setCookie] = function (msg) {
    var parsedUrl = _utilsUrl.parseProxyUrl(msg.url);
    var cookieUrl = parsedUrl ? parsedUrl.destUrl : msg.url;

    this.cookies.setByClient(cookieUrl, msg.cookie);

    return this.cookies.getClientString(cookieUrl);
};

ServiceMessages[_command2.default.uploadFiles] = function callee$0$0(msg) {
    return _regeneratorRuntime.async(function callee$0$0$(context$1$0) {
        while (1) switch (context$1$0.prev = context$1$0.next) {
            case 0:
                context$1$0.next = 2;
                return _regeneratorRuntime.awrap(this.uploadStorage.store(msg.fileNames, msg.data));

            case 2:
                return context$1$0.abrupt('return', context$1$0.sent);

            case 3:
            case 'end':
                return context$1$0.stop();
        }
    }, null, this);
};

ServiceMessages[_command2.default.getUploadedFiles] = function callee$0$0(msg) {
    return _regeneratorRuntime.async(function callee$0$0$(context$1$0) {
        while (1) switch (context$1$0.prev = context$1$0.next) {
            case 0:
                context$1$0.next = 2;
                return _regeneratorRuntime.awrap(this.uploadStorage.get(msg.filePaths));

            case 2:
                return context$1$0.abrupt('return', context$1$0.sent);

            case 3:
            case 'end':
                return context$1$0.stop();
        }
    }, null, this);
};
module.exports = exports.default;