var _interopRequireDefault = require('babel-runtime/helpers/interop-require-default').default;

exports.__esModule = true;
exports.isSubDomain = isSubDomain;
exports.sameOriginCheck = sameOriginCheck;
exports.convertHostToLowerCase = convertHostToLowerCase;
exports.getProxyUrl = getProxyUrl;
exports.getDomain = getDomain;
exports.parseProxyUrl = parseProxyUrl;
exports.getPathname = getPathname;
exports.parseUrl = parseUrl;
exports.isSupportedProtocol = isSupportedProtocol;
exports.resolveUrlAsDest = resolveUrlAsDest;
exports.formatUrl = formatUrl;
exports.prepareUrl = prepareUrl;
exports.ensureTrailingSlash = ensureTrailingSlash;
// -------------------------------------------------------------
// WARNING: this file is used by both the client and the server.
// Do not use any browser or node-specific API!
// -------------------------------------------------------------

var _stringTrim = require('./string-trim');

var _stringTrim2 = _interopRequireDefault(_stringTrim);

//Const
var PROTOCOL_RE = /(^(\w+?\:))/;
var LEADING_SLASHES_RE = /^(\/\/)/;
var HOST_RE = /^(.*?)(\/|%|\?|;|#|$)/;
var PORT_RE = /:([0-9]*)$/;
var QUERY_AND_HASH_RE = /(\?.+|#[^#]*)$/;

var URL_UTIL_PROTOCOL_IS_NOT_SUPPORTED = 'CLIENT_URL_UTIL_PROTOCOL_IS_NOT_SUPPORTED';
exports.URL_UTIL_PROTOCOL_IS_NOT_SUPPORTED = URL_UTIL_PROTOCOL_IS_NOT_SUPPORTED;
var REQUEST_DESCRIPTOR_VALUES_SEPARATOR = '!';
exports.REQUEST_DESCRIPTOR_VALUES_SEPARATOR = REQUEST_DESCRIPTOR_VALUES_SEPARATOR;
var IFRAME = 'iframe';
exports.IFRAME = IFRAME;
var SCRIPT = 'script';

exports.SCRIPT = SCRIPT;
function validateDestUrl(url) {
    if (!/^https?:/.test(url)) {
        throw {
            code: URL_UTIL_PROTOCOL_IS_NOT_SUPPORTED,
            destUrl: url
        };
    }
}

function isSubDomain(domain, subDomain) {
    domain = domain.replace(/^www./i, '');
    subDomain = subDomain.replace(/^www./i, '');

    if (domain === subDomain) return true;

    var index = subDomain.lastIndexOf(domain);

    return subDomain[index - 1] === '.' && subDomain.length === index + domain.length;
}

function sameOriginCheck(location, checkedUrl) {
    if (!checkedUrl) return true;

    var parsedLocation = parseUrl(location);
    var parsedCheckedUrl = parseUrl(checkedUrl);
    var parsedProxyLocation = parseProxyUrl(location);
    var parsedDestUrl = parsedProxyLocation ? parsedProxyLocation.destResourceInfo : parsedLocation;
    var isRelative = !parsedCheckedUrl.host;

    if (isRelative || parsedCheckedUrl.host === parsedLocation.host && parsedCheckedUrl.protocol === parsedLocation.protocol) return true;

    if (parsedDestUrl) {
        var portsEq = !parsedDestUrl.port && !parsedCheckedUrl.port || parsedDestUrl.port && parsedDestUrl.port.toString() === parsedCheckedUrl.port;

        if (parsedDestUrl.protocol === parsedCheckedUrl.protocol && portsEq) {
            if (parsedDestUrl.hostname === parsedCheckedUrl.hostname) return true;

            return isSubDomain(parsedDestUrl.hostname, parsedCheckedUrl.hostname) || isSubDomain(parsedCheckedUrl.hostname, parsedDestUrl.hostname);
        }
    }

    return false;
}

// NOTE: Convert the destination protocol and hostname to the lower case. (GH-1)

function convertHostToLowerCase(url) {
    var parsedUrl = parseUrl(url);

    return (parsedUrl.protocol + '//' + parsedUrl.host).toLowerCase() + parsedUrl.partAfterHost;
}

function getProxyUrl(url, proxyHostname, proxyPort, sessionId, resourceType, charset) {
    validateDestUrl(url);

    var params = [sessionId];

    if (resourceType) params.push(resourceType);

    if (charset) params.push(charset);

    params = params.join(REQUEST_DESCRIPTOR_VALUES_SEPARATOR);

    return 'http://' + proxyHostname + ':' + proxyPort + '/' + params + '/' + convertHostToLowerCase(url);
}

function getDomain(parsed) {
    return formatUrl({
        protocol: parsed.protocol,
        host: parsed.host,
        hostname: parsed.hostname,
        port: parsed.port
    });
}

function parseProxyUrl(proxyUrl) {
    // TODO: Remove it.
    var parsedUrl = parseUrl(proxyUrl);

    if (!parsedUrl.partAfterHost) return null;

    var match = parsedUrl.partAfterHost.match(/^\/(\S+?)\/(https?:\/\/\S+)/);

    if (!match) return null;

    var params = match[1].split(REQUEST_DESCRIPTOR_VALUES_SEPARATOR);

    // NOTE: We should have, at least, the job uid and the owner token.
    if (!params.length) return null;

    return {
        destUrl: match[2],
        destResourceInfo: parseUrl(match[2]),
        partAfterHost: parsedUrl.partAfterHost,

        proxy: {
            hostname: parsedUrl.hostname,
            port: parsedUrl.port
        },

        sessionId: params[0],
        resourceType: params[1] || null,
        charset: params[2] || null
    };
}

function getPathname(path) {
    return path.replace(QUERY_AND_HASH_RE, '');
}

function parseUrl(url) {
    var parsed = {};

    url = prepareUrl(url);

    if (!url) return parsed;

    url = _stringTrim2.default(url);

    // Protocol
    var hasImplicitProtocol = false;
    var remainder = url.replace(PROTOCOL_RE, function (str, protocol) {
        parsed.protocol = protocol;
        return '';
    }).replace(LEADING_SLASHES_RE, function () {
        hasImplicitProtocol = true;
        return '';
    });

    // NOTE: the URL is relative.
    if (!parsed.protocol && !hasImplicitProtocol) {
        parsed.partAfterHost = url;
        return parsed;
    }

    // Host
    parsed.partAfterHost = remainder.replace(HOST_RE, function (str, host, restPartSeparator) {
        parsed.host = host;
        return restPartSeparator;
    });

    if (parsed.host) {
        parsed.hostname = parsed.host.replace(PORT_RE, function (str, port) {
            parsed.port = port;
            return '';
        });
    }

    return parsed;
}

function isSupportedProtocol(url) {
    return !/^\s*(chrome-extension:|blob:|javascript:|about:|mailto:|tel:|data:|skype:|skypec2c:|file:|#)/i.test(url);
}

function resolveUrlAsDest(url, getProxyUrlMeth) {
    getProxyUrlMeth = getProxyUrlMeth || getProxyUrl;

    if (isSupportedProtocol(url)) {
        var proxyUrl = getProxyUrlMeth(url);
        var parsedProxyUrl = parseProxyUrl(proxyUrl);

        return formatUrl(parsedProxyUrl.destResourceInfo);
    }

    return url;
}

function formatUrl(parsedUrl) {
    // NOTE: the URL is relative.
    if (!parsedUrl.host && (!parsedUrl.hostname || !parsedUrl.port)) return parsedUrl.partAfterHost;

    var url = parsedUrl.protocol || '';

    url += '//';

    if (parsedUrl.username || parsedUrl.password) url += parsedUrl.username + ':' + parsedUrl.password + '@';

    if (parsedUrl.host) url += parsedUrl.host;else {
        url += parsedUrl.hostname;

        if (parsedUrl.port) url += ':' + parsedUrl.port;
    }

    if (parsedUrl.partAfterHost) url += parsedUrl.partAfterHost;

    return url;
}

function prepareUrl(url) {
    // TODO: fix it
    /* eslint-disable no-undef */
    if (url === null && /iPad|iPhone/i.test(window.navigator.userAgent)) return '';
    /* eslint-enable no-undef */

    url = (url + '').replace(/\n|\t/g, '');

    // NOTE: Remove unnecessary slashes from the beginning of the url.
    // For example, the "//////google.com" url is equal to "//google.com".
    return url.replace(/^\/+(\/\/.*$)/, '$1');
}

function ensureTrailingSlash(srcUrl, processedUrl) {
    var hasTrailingSlash = /\/$/.test(srcUrl);

    if (!hasTrailingSlash) processedUrl = processedUrl.replace(/\/$/, '');

    return processedUrl;
}